-- BGET benchmark program
-- Standard Euphoria routines and BGET routines

include file.e
include get.e
include machine.e
tick_rate(1000)

include bget.e

constant BGET_DAT = "_bget.dat", GET_DAT = "_get.dat"

integer iter, fn
sequence file, s
object junk

procedure bench()
atom t, p, g, bp, bg
   
   printf(1, "Benchmark results, input file %s, %d iterations:\n\n", {file, iter})
   
   -- standard Euphoria print()
   fn = open(GET_DAT, "w")
   t = time()
   for i = 1 to iter do
      junk = seek(fn, 0)
      print(fn, s)
   end for
   p = time() - t
   printf(1, "print     %f\n", p)
   close(fn)
   
   -- bprint()
   fn = open(BGET_DAT, "wb")
   t = time()
   for i = 1 to iter do
      junk = seek(fn, 0)
      bprint(fn, s)
   end for
   bp = time() - t
   printf(1, "bprint    %f\n\n", bp)
   close(fn)
   
   
   -- standard Euphoria get()
   fn = open(GET_DAT, "r")
   t = time()
   for i = 1 to iter do
      junk = seek(fn, 0)
      s = get(fn)
   end for
   g = time() - t
   printf(1, "get       %f\n", g)
   close(fn)
   
   -- bget()
   fn = open(BGET_DAT, "rb")
   t = time()
   for i = 1 to iter do
      junk = seek(fn, 0)
      s = bget(fn)
   end for
   bg = time() - t
   printf(1, "bget      %f\n\n", bg)
   close(fn)

   printf(1, "print/get total time:   %f\n",   p+g)
   printf(1, "bprint/bget total time: %f\n\n", bp+bg)


   -- file sizes
   s = dir(file)
   printf(1, "%-12s size: %10d\n", {file, s[1][D_SIZE]})
   s = dir(GET_DAT)
   printf(1, "%-12s size: %10d\n", {GET_DAT, s[1][D_SIZE]})
   s = dir(BGET_DAT)
   printf(1, "%-12s size: %10d\n", {BGET_DAT, s[1][D_SIZE]})
end procedure

type positive_atom(object x)
   if atom(x) then
      return x > 0
   end if
   return 0
end type

procedure main()
sequence cmd, t
   file = "btest.dat"
   iter = 5000
   
   cmd = command_line()
   cmd = cmd[3..length(cmd)]
   if length(cmd) then
      file = cmd[1]
      if length(cmd) >= 2 then
	 t = value(cmd[2])
	 if t[1] != GET_SUCCESS or not positive_atom(t[2]) then
	    printf(2, "%s is not a valid number of iterations\n", {cmd[2]})
	 else
	    iter = t[2]
	 end if
      end if
   end if
   
   fn = open(file, "r")
   if fn = -1 then
      printf(2, "%s cannot be opened for read!\n", {file})
   else
      s = get(fn)
      close(fn)
      if s[1] != GET_SUCCESS then
	 printf(2, "%s is corrupted, or does not contain sequence data.\n", {file})
      else
	 s = s[2]
	 fn = open(BGET_DAT, "wb")
	 if fn = -1 then
	    printf(2, "%s cannot be opened for write!\n", {BGET_DAT})
	 else
	    bprint(fn, s)
	    close(fn)
	    fn = open(BGET_DAT, "rb")
	    if fn = -1 then
	       printf(2, "%s cannot be opened for read!\n", {BGET_DAT})
	    else
	       t = bget(fn)
	       close(fn)
	       if t[1] != GET_SUCCESS then
		  printf(1, "Argh, %s is corrupted!  Alert Gabriel!\n", {BGET_DAT})
		  ? t
	       else
		  if not equal(s, t[2]) then
		     printf(1, "Argh, %s contained incorrect data!  Alert Gabriel!\n", {BGET_DAT})
		     ? s
		     ? t[2]
		  else
		     bench()
		  end if
	       end if
	    end if
	 end if
      end if
   end if
end procedure

main()
while get_key() != -1 do
   -- clear keyboard buffer
end while
puts(2, "\nPress any key to continue . . .")
if wait_key() then
   -- wait for keypress before ending
   puts(2, "\n")
end if

